#
# 2202.05924-fishing.R, 13 Mar 22
#
# Data:
# Parameter, Compute and Data Trends in Machine Learning
# Jaime Sevilla, Pablo Villalobos, Juan Felipe Cerón, Matthew Burtell, Lennart Heim, Amogh B. Nanjajjar, Anson Ho, Tamay Besiroglu and Marius Hobbhahn
#

library("colorspace")

pal_col=rainbow(3)
par(bty="l")
par(las=1)
par(pch="+")


ml=read.csv("2202.05924_ParameterComputeDataTrends.csv", as.is=TRUE)

# Lots of NAs
ml=subset(ml, !is.na(Training.compute..FLOPs.))

just_year=(nchar(ml$Publication.date) == 4)
day_missing=(nchar(ml$Publication.date) == 7)
ml$Publication.date[just_year]=paste0("01/06/", ml$Publication.date[just_year])
ml$Publication.date[day_missing]=paste0("15/", ml$Publication.date[day_missing])

ml$pub_date=as.Date(ml$Publication.date, format="%d/%m/%Y")
base_date=as.Date("01/01/1985", format="%d/%m/%Y")
AWS_date=as.Date("01/01/2008", format="%d/%m/%Y")

b_AWS=subset(ml, pub_date <  AWS_date)
a_AWS=subset(ml, pub_date >= AWS_date)

AWS=subset(a_AWS, Parameters <= 5e10)
bespoke=subset(a_AWS, Parameters > 5e10)

pal_col=rainbow(3)
plot(base_date, 1, type="n", log="y",
	xlim=c(base_date, max(ml$pub_date, na.rm=TRUE)),
			 ylim=c(1e7, max(ml$Training.compute..FLOPs., na.rm=TRUE)),
	xlab="Date", ylab="Training FLOPS")
points(b_AWS$pub_date, b_AWS$Training.compute..FLOPs., col=pal_col[3])
b_AWS_mod=glm(log(Training.compute..FLOPs.) ~ pub_date, data=b_AWS)
# summary(b_AWS_mod)
pred=predict(b_AWS_mod)
lines(b_AWS$pub_date, exp(pred), col=pal_col[3])
points(AWS$pub_date, AWS$Training.compute..FLOPs., col=pal_col[2])
AWS_mod=glm(log(Training.compute..FLOPs.) ~ pub_date, data=AWS)
# summary(AWS_mod)
pred=predict(AWS_mod)
lines(AWS$pub_date, exp(pred), col=pal_col[2])
points(bespoke$pub_date, bespoke$Training.compute..FLOPs., col=pal_col[1])

legend("topleft", legend=c("Bespoke", "Cloud", "Do-it yourself"), bty="n", fill=pal_col)

# Number of model parameters over time
pal_col=rainbow(2)
plot(base_date, 1, type="n", log="y",
	xlim=c(AWS_date, max(ml$pub_date, na.rm=TRUE)), ylim=c(1e7, 5e13),
	xlab="Date", ylab="Model parameters")
points(AWS$pub_date, AWS$Parameters, col=pal_col[2])
points(bespoke$pub_date, bespoke$Parameters, col=pal_col[1])


